/* $Id$ */
/*
 * functions to make X11 BDF font bold.
 */
#include "common.h"
#include "bold.h"

static void hex_get_line(int d[], const char *s, size_t len);
static void shl(int dout[], int din[], size_t size);
static void shr(int dout[], int din[], size_t size);
static void bitcalc(int out[], int d1[], int d2[], int d3[], size_t size);


/*
 * recast dummy for font property.
 */
void recast_dummy(char *dst, int *padding, int *width, int *height, int *dx, int dy, const char *src)
{
  sscanf(src, "%*s%d%d%*d%*d", width, height);
  strcpy(dst, src);
}

/*
 * output BITMAP image.
 */
void hex_output(FILE *fp, int d[], size_t n)
{
  size_t i;

  for (i = 0; i < n; i++) {
    fprintf(fp, "%02x", d[i]);
  }
  fprintf(fp, "\n");
}

/*
 * create bold font BITMAP image.
 */
void hex_make_bold(int d[], const char *s, size_t len, int bdir, int pile)
{
  int *bold;
  int *tmp;

  bold = XMALLOC(int, len);
  tmp = XMALLOC(int, len);
  if (bdir) {
    /* left shifted image */
    hex_get_line(d, s, len);
    shl(bold, d, len);
  } else {
    /* right shifted image */
    hex_get_line(bold, s, len);
    shr(d, bold, len);
  }
  if (pile) {
    /* left edge */
    shl(tmp, bold, len);
    bitcalc(d, tmp, d, bold, len);
  } else {
    /* right edge */
    shr(tmp, d, len);
    bitcalc(d, tmp, bold, d, len);
  }
  XFREE(bold);
  XFREE(tmp);
}

static void hex_get_line(int d[], const char *s, size_t len)
{
  unsigned int b;
  size_t i;

  for (i = 0; i < len; i++) {
    sscanf(s, "%02x", &b);
    d[i] = b;
    s += 2;
  }
}

/*
 * shift left.
 */
static void shl(int dout[], int din[], size_t size)
{
  int c, d;
  int i;

  c = 0x00;
  for (i = size-1; i >= 0; i--) {
    d = c;
    c = (din[i] & 0x80) >> 7;
    dout[i] = ((din[i] << 1) & 0xff) | d;
  }
}

/*
 * shift right.
 */
static void shr(int dout[], int din[], size_t size)
{
  int c, d;
  size_t i;

  c = 0x00;
  for (i = 0; i < size; i++) {
    d = c;
    c = (din[i] & 0x01) << 7;
    dout[i] = (din[i] >> 1) | d;
  }
}

/*
 * bit operation.
 */
static void bitcalc(int out[], int d1[], int d2[], int d3[], size_t size)
{
  size_t i;

  for (i = 0; i < size; i++) {
    out[i] = (~d1[i] & d2[i]) | d3[i];
  }
}
