/* $Id$ */
/*
 * makes X11 BDF font bold.
 */
#include <ctype.h>
#include "common.h"
#include "bold.h"
#include "utils.h"

#define BUFLEN 256

static void mkbold(int bdir, int pile, int verbose, char *filename);
static void get_option(int *bdir, int *pile, int *verbose, char **filename, int argc, char *argv[]);
static void usage(FILE *fp, int status);


int main(int argc, char *argv[])
{
  int bdir = TRUE;  /* left shifted image */
  int pile = FALSE; /* right edge */
  int verbose = FALSE;
  char *filename = NULL;

  get_option(&bdir, &pile, &verbose, &filename, argc, argv);
  mkbold(bdir, pile, verbose, filename);
  return EXIT_SUCCESS;
}

static void mkbold(int bdir, int pile, int verbose, char *filename)
{
  FILE *fp;
  char buf[BUFLEN], keyword[BUFLEN];
  Property prop;
  size_t len;
  int *d;
  int width, height;
  int bitmap;
  ProgressMeter *meter;
  int max_chars;

  fp = file_open(filename);
  prop.weight[0] = "Medium";
  prop.weight[1] = "Bold";
  prop.slant[0] = prop.slant[1] = NULL;
  prop.width = &width;
  prop.height = &height;
  prop.recast = recast_dummy;
  max_chars = replace_property(fp, BUFLEN, &prop);
  if (max_chars <= VERBOSE_MIN) {
    verbose = FALSE;
  }
  if (verbose) {
    meter = progress_meter_new(max_chars);
  }
  len = (width-1)/8+1;
  d = XMALLOC(int, len);
  bitmap = FALSE;
  while (fgets(buf, BUFLEN, fp) != NULL) {
    if (isalpha(buf[0])) {
      sscanf(buf, "%s", keyword);
      if (strcmp(keyword, "BITMAP") == 0) {
        bitmap = TRUE;
        fputs(buf, stdout);
        continue;
      }
      if (strcmp(keyword, "ENDCHAR") == 0) {
        bitmap = FALSE;
        fputs(buf, stdout);
        if (verbose) {
          progress_meter_print(meter);
        }
        continue;
      }
    }

    if (bitmap) {
      hex_make_bold(d, buf, len, bdir, pile);
      hex_output(stdout, d, len);
      continue;
    }
    fputs(buf, stdout);
  }
  fclose(fp);

  if (verbose) {
    progress_meter_clear(meter);
    progress_meter_delete(meter);
  }
  XFREE(d);
}

static void get_option(int *bdir, int *pile, int *verbose, char **filename, int argc, char *argv[])
{
  int i;

  for (i = 1; i < argc; i++) {
    if (*argv[i] == '-') {
      switch (*(argv[i]+1)) {
      case 'r':
        *bdir = FALSE;
        break;
      case 'l':
        *bdir = TRUE;
        break;
      case 'R':
        *pile = FALSE;
        break;
      case 'L':
        *pile = TRUE;
        break;
      case 'V':
        *verbose = TRUE;
        break;
      case 'h':
        usage(stdout, EXIT_SUCCESS);
        break;
      }
    } else {
      *filename = argv[i];
      break;
    }
  }
}

static void usage(FILE *fp, int status)
{
  fprintf(fp,
          "Usage: mkbold [OPTION] [FILE]\n\n"
          "Write processed X11 BDF file to standard output.\n"
          "With no FILE, read standard input.\n"
          "\nOptions:\n"
          "  -l left shifted image OR original (default)\n"
          "  -r right shifted image OR original\n"
          "  -L reflect left edge\n"
          "  -R reflect right edge (default)\n"
          "  -V verbose output\n"
          "  -h display this help and exit\n"
          );
  exit(status);
}
