/* $Id$ */
/*
 * makes X11 BDF font bold-italic.
 */
#include <ctype.h>
#include "common.h"
#include "bold.h"
#include "italic.h"
#include "bolditalic.h"
#include "utils.h"

#define BUFLEN 256

static void mkbolditalic(int bdir, int pile, int correct, int dy, int verbose, char *filename);
static void get_option(int *bdir, int *pile, int *correct, int *dy, int *verbose, char **filename, int argc, char *argv[]);
static void usage(FILE *fp, int status);


int main(int argc, char *argv[])
{
  int bdir = TRUE;  /* left shifted image */
  int pile = FALSE; /* right edge */
  int correct = PIXEL_CORRECTION;
  int dy = SLANT_DEEP;
  int verbose = FALSE;
  char *filename = NULL;

  get_option(&bdir, &pile, &correct, &dy, &verbose, &filename, argc, argv);
  mkbolditalic(bdir, pile, correct, dy, verbose, filename);
  return EXIT_SUCCESS;
}

static void mkbolditalic(int bdir, int pile, int correct, int dy, int verbose, char *filename)
{
  FILE *fp;
  char buf[BUFLEN], outbuf[BUFLEN], keyword[BUFLEN];
  Property prop;
  Pattern *ptable;
  Bitmap *bmap;
  size_t count;
  int width, height, dx, padding;
  int bitmap;
  ProgressMeter *meter;
  int max_chars;

  fp = file_open(filename);
  ptable = pattern_new(correct);
  prop.weight[0] = "Medium";
  prop.weight[1] = "Bold";
  prop.slant[0] = "R";
  prop.slant[1] = "I";
  prop.padding = &padding;
  prop.width = &width;
  prop.height = &height;
  prop.dx = &dx;
  prop.dy = dy;
  prop.recast = recast_bbx;
  max_chars = replace_property(fp, BUFLEN, &prop);
  if (max_chars <= VERBOSE_MIN) {
    verbose = FALSE;
  }
  if (verbose) {
    meter = progress_meter_new(max_chars);
  }
  bmap = bitmap_new(height, dx-1+width+padding);
  count = 0;
  bitmap = FALSE;
  while (fgets(buf, BUFLEN, fp) != NULL) {
    if (isalpha(buf[0])) {
      sscanf(buf, "%s", keyword);
      if (strcmp(keyword, "BBX") == 0) {
        /* recast the metric information from the bounding box */
        recast_bbx(outbuf, &padding, &width, &height, &dx, dy, buf);
        fputs(outbuf, stdout);
        continue;
      }
      if (strcmp(keyword, "BITMAP") == 0) {
        bitmap = TRUE;
        count = 0;
        fputs(buf, stdout);
        continue;
      }
      if (strcmp(keyword, "ENDCHAR") == 0) {
        bitmap = FALSE;
        if (correct > 0) {
          bitmap_correct_pixel(bmap, ptable, width, height, dy);
        }
        bitmap_make_slant(bmap, width, height, dx, dy, padding);
        /* now thinking! */
        bitmap_make_bold_output(stdout, bmap, width+dx-1, count, bdir, pile);
        /*
        bitmap_output(stdout, bmap, height);
        hex_make_bold(d, buf, len, bdir, pile);
        hex_output(stdout, d, len);
        */
        fputs(buf, stdout);
        if (verbose) {
          progress_meter_print(meter);
        }
        continue;
       }
    }

    if (bitmap) {
      bitmap_get_line(bmap, count, buf);
      count++;
      continue;
    }
    fputs(buf, stdout);
  }
  fclose(fp);

  if (verbose) {
    progress_meter_clear(meter);
    progress_meter_delete(meter);
  }
  pattern_delete(ptable);
  bitmap_delete(bmap);
}

static void get_option(int *bdir, int *pile, int *correct, int *dy, int *verbose, char **filename, int argc, char *argv[])
{
  int i;

  for (i = 1; i < argc; i++) {
    if (*argv[i] == '-') {
      switch (*(argv[i]+1)) {
      case 'r':
        *bdir = FALSE;
        break;
      case 'l':
        *bdir = TRUE;
        break;
      case 'R':
        *pile = FALSE;
        break;
      case 'L':
        *pile = TRUE;
        break;
#if defined DEBUG
      case 'c':
        *correct = atoi(argv[i]+2);
        break;
      case 's':
        *dy = atoi(argv[i]+2);
        break;
#endif /* DEBUG */
      case 'V':
        *verbose = TRUE;
        break;
      case 'h':
        usage(stdout, EXIT_SUCCESS);
        break;
      }
    } else {
      *filename = argv[i];
      break;
    }
  }
}

static void usage(FILE *fp, int status)
{
  fprintf(fp,
          "Usage: mkbolditalic [OPTION] [FILE]\n\n"
          "Write processed X11 BDF file to standard output.\n"
          "With no FILE, read standard input.\n"
          "\nOptions:\n"
          "  -l left shifted image OR original (default)\n"
          "  -r right shifted image OR original\n"
          "  -L reflect left edge\n"
          "  -R reflect right edge (default)\n"
          "  -V verbose output\n"
          "  -h display this help and exit\n"
          );
  exit(status);
}
