/* $Id$ */
/*
 * makes X11 BDF font italic.
 */
#include <ctype.h>
#include "common.h"
#include "italic.h"
#include "utils.h"

#define BUFLEN 256

static void mkitalic(int correct, int dy, int verbose, char *filename);
static void get_option(int *correct, int *dy, int *verbose, char **filename, int argc, char *argv[]);
static void usage(FILE *fp, int status);


int main(int argc, char *argv[])
{
  int correct = PIXEL_CORRECTION;
  int dy = SLANT_DEEP;
  int verbose = FALSE;
  char *filename = NULL;

  get_option(&correct, &dy, &verbose, &filename, argc, argv);
  mkitalic(correct, dy, verbose, filename);
  return EXIT_SUCCESS;
}

static void mkitalic(int correct, int dy, int verbose, char *filename)
{
  FILE *fp;
  char buf[BUFLEN], outbuf[BUFLEN], keyword[BUFLEN];
  Property prop;
  Pattern *ptable;
  Bitmap *bmap;
  size_t count;
  int width, height, dx, padding;
  int bitmap;
  ProgressMeter *meter;
  int max_chars;

  fp = file_open(filename);
  count = 0;
  ptable = pattern_new(correct);
  prop.weight[0] = prop.weight[1] = NULL;
  prop.slant[0] = "R";
  prop.slant[1] = "I";
  prop.padding = &padding;
  prop.width = &width;
  prop.height = &height;
  prop.dx = &dx;
  prop.dy = dy;
  prop.recast = recast_bbx;
  max_chars = replace_property(fp, BUFLEN, &prop);
  if (max_chars <= VERBOSE_MIN) {
    verbose = FALSE;
  }
  if (verbose) {
    meter = progress_meter_new(max_chars);
  }
  bmap = bitmap_new(height, dx-1+width+padding);
  bitmap = FALSE;
  while (fgets(buf, BUFLEN, fp) != NULL) {
    if (isalpha(buf[0])) {
      sscanf(buf, "%s", keyword);
      if (strcmp(keyword, "BBX") == 0) {
        /* recast the metric information from the bounding box */
        recast_bbx(outbuf, &padding, &width, &height, &dx, dy, buf);
        fputs(outbuf, stdout);
        continue;
      }
      if (strcmp(keyword, "BITMAP") == 0) {
        bitmap = TRUE;
        count = 0;
        fputs(buf, stdout);
        continue;
      }
      if (strcmp(keyword, "ENDCHAR") == 0) {
        bitmap = FALSE;
        if (correct > 0) {
          bitmap_correct_pixel(bmap, ptable, width, height, dy);
        }
        bitmap_make_slant(bmap, width, height, dx, dy, padding);
        bitmap_output(stdout, bmap, height);
        fputs(buf, stdout);
        if (verbose) {
          progress_meter_print(meter);
        }
        continue;
      }
    }

    if (bitmap) {
      bitmap_get_line(bmap, count, buf);
      count++;
      continue;
    }
    fputs(buf, stdout);
  }
  fclose(fp);

  if (verbose) {
    progress_meter_clear(meter);
    progress_meter_delete(meter);
  }
  pattern_delete(ptable);
  bitmap_delete(bmap);
}

static void get_option(int *correct, int *dy, int *verbose, char **filename, int argc, char *argv[])
{
  int i;

  for (i = 1; i < argc; i++) {
    if (*argv[i] == '-') {
      switch (*(argv[i]+1)) {
#if defined DEBUG
      case 'c':
        *correct = atoi(argv[i]+2);
        break;
      case 's':
        *dy = atoi(argv[i]+2);
        break;
#endif /* DEBUG */
      case 'V':
        *verbose = TRUE;
        break;
      case 'h':
        usage(stdout, EXIT_SUCCESS);
        break;
      }
    } else {
      *filename = argv[i];
      break;
    }
  }
}

static void usage(FILE *fp, int status)
{
  fprintf(fp,
          "Usage: mkitalic [OPTION] [FILE]\n\n"
          "Write processed X11 BDF file to standard output.\n"
          "With no FILE, read standard input.\n"
          "\nOptions:\n"
          "  -V verbose output\n"
          "  -h display this help and exit\n"
          );
  exit(status);
}
